#
# Copyright (c) nexB Inc. and others. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# See http://www.apache.org/licenses/LICENSE-2.0 for the license text.
# See https://github.com/nexB/commoncode for support or download.
# See https://aboutcode.org for more information about nexB OSS projects.
#

import os
from os.path import join
from os.path import sep
from unittest.case import skip
from unittest.case import skipIf

from commoncode import filetype
from commoncode import fileutils
from commoncode.fileutils import as_posixpath
from commoncode.system import on_linux
from commoncode.system import on_posix
from commoncode.system import on_mac
from commoncode.system import on_macos_14_or_higher
from commoncode.system import on_windows
from commoncode.testcase import FileBasedTesting
from commoncode.testcase import make_non_executable
from commoncode.testcase import make_non_readable
from commoncode.testcase import make_non_writable


@skip('Somehow permissions tests do not work OK yet on Python 3')
class TestPermissionsDeletions(FileBasedTesting):
    """
    This is failing for now on Python 3
    """
    test_data_dir = os.path.join(os.path.dirname(__file__), 'data')

    def test_delete_unwritable_directory_and_files(self):
        base_dir = self.get_test_loc('fileutils/readwrite', copy=True)
        test_dir = join(base_dir, 'sub')
        test_file = join(test_dir, 'file')

        try:
            # note: there are no unread/writable dir on windows
            make_non_readable(test_file)
            make_non_executable(test_file)
            make_non_writable(test_file)

            make_non_readable(test_dir)
            make_non_executable(test_dir)
            make_non_writable(test_dir)

            fileutils.delete(test_dir)
            assert not os.path.exists(test_dir)
        finally:
            fileutils.chmod(base_dir, fileutils.RW, recurse=True)


class TestPermissions(FileBasedTesting):
    """
    Several assertions or test are skipped on non posix OSes.
    Windows handles permissions and special files differently.
    """
    test_data_dir = os.path.join(os.path.dirname(__file__), 'data')

    def test_chmod_on_non_existing_file_throws_no_exception(self):
        fileutils.chmod('some non existing dir', fileutils.RWX)

    def test_chmod_read_write_recursively_on_dir(self):
        test_dir = self.get_test_loc('fileutils/executable', copy=True)
        test_file = join(test_dir, 'deep1', 'deep2', 'ctags')
        test_dir2 = join(test_dir, 'deep1', 'deep2')
        parent = join(test_dir, 'deep1')

        try:
            make_non_writable(test_file)
            assert not filetype.is_writable(test_file)
            if on_posix:
                make_non_executable(test_file)
                assert not filetype.is_executable(test_file)

            if on_posix:
                make_non_executable(test_dir2)
                assert not filetype.is_executable(test_dir2)
            make_non_writable(test_dir)
            if on_posix:
                assert not filetype.is_writable(test_dir2)

            fileutils.chmod(parent, fileutils.RW, recurse=True)

            assert filetype.is_readable(test_dir2) is True
            assert filetype.is_writable(test_dir2)
            if on_posix:
                assert filetype.is_executable(test_dir2)
        finally:
            fileutils.chmod(test_dir, fileutils.RW, recurse=True)

    def test_chmod_read_write_non_recursively_on_dir(self):
        test_dir = self.get_test_loc('fileutils/executable', copy=True)
        test_file = join(test_dir, 'deep1', 'deep2', 'ctags')
        test_dir = join(test_dir, 'deep1', 'deep2')
        parent = join(test_dir, 'deep1')

        try:
            # setup
            make_non_writable(test_file)
            assert not filetype.is_writable(test_file)

            make_non_writable(test_dir)
            if on_posix:
                assert not filetype.is_writable(test_dir)
            else:
                # windows is different
                assert filetype.is_writable(test_dir)

            fileutils.chmod(parent, fileutils.RW, recurse=False)
            # test: the perms should be the same
            assert not filetype.is_writable(test_file)

            if on_posix:
                assert not filetype.is_writable(test_dir)
            else:
                # windows is different
                assert filetype.is_writable(test_dir)
        finally:
            fileutils.chmod(test_dir, fileutils.RW, recurse=True)

    def test_chmod_read_write_file(self):
        test_dir = self.get_test_loc('fileutils/executable', copy=True)
        test_file = join(test_dir, 'deep1', 'deep2', 'ctags')

        try:
            make_non_writable(test_file)
            assert not filetype.is_writable(test_file)

            fileutils.chmod(test_file, fileutils.RW)
            assert filetype.is_readable(test_file)
            assert filetype.is_writable(test_file)
        finally:
            fileutils.chmod(test_dir, fileutils.RW, recurse=True)

    def test_chmod_read_write_exec_dir(self):
        test_dir = self.get_test_loc('fileutils/executable', copy=True)
        test_file = join(test_dir, 'deep1', 'deep2', 'ctags')

        try:
            if on_posix:
                make_non_executable(test_dir)
                assert not filetype.is_executable(test_file)
            make_non_writable(test_dir)

            fileutils.chmod(test_dir, fileutils.RWX, recurse=True)
            assert filetype.is_readable(test_file)
            assert filetype.is_writable(test_file)
            if on_posix:
                assert filetype.is_executable(test_file)
        finally:
            fileutils.chmod(test_dir, fileutils.RW, recurse=True)

    def test_copyfile_does_not_keep_permissions(self):
        src_file = self.get_temp_file()
        dest = self.get_temp_dir()
        with open(src_file, 'w') as f:
            f.write(u'')
        try:
            make_non_readable(src_file)
            if on_posix:
                assert not filetype.is_readable(src_file)

            fileutils.copyfile(src_file, dest)
            dest_file = join(dest, os.listdir(dest)[0])
            assert filetype.is_readable(dest_file)
        finally:
            fileutils.chmod(src_file, fileutils.RW, recurse=True)
            fileutils.chmod(dest, fileutils.RW, recurse=True)

    def test_copytree_does_not_keep_non_writable_permissions(self):
        src = self.get_test_loc('fileutils/exec', copy=True)
        dst = self.get_temp_dir()

        try:
            src_file = join(src, 'subtxt/a.txt')
            make_non_writable(src_file)
            assert not filetype.is_writable(src_file)

            src_dir = join(src, 'subtxt')
            make_non_writable(src_dir)
            if on_posix:
                assert not filetype.is_writable(src_dir)

            # copy proper
            dest_dir = join(dst, 'dest')
            fileutils.copytree(src, dest_dir)

            dst_file = join(dest_dir, 'subtxt/a.txt')
            assert os.path.exists(dst_file)
            assert filetype.is_writable(dst_file)

            dest_dir2 = join(dest_dir, 'subtxt')
            assert os.path.exists(dest_dir2)
            assert filetype.is_writable(dest_dir)
        finally:
            fileutils.chmod(src, fileutils.RW, recurse=True)
            fileutils.chmod(dst, fileutils.RW, recurse=True)

    def test_copytree_copies_unreadable_files(self):
        src = self.get_test_loc('fileutils/exec', copy=True)
        dst = self.get_temp_dir()
        src_file1 = join(src, 'a.bat')
        src_file2 = join(src, 'subtxt', 'a.txt')

        try:
            # make some unreadable source files
            make_non_readable(src_file1)
            if on_posix:
                assert not filetype.is_readable(src_file1)

            make_non_readable(src_file2)
            if on_posix:
                assert not filetype.is_readable(src_file2)

            # copy proper
            dest_dir = join(dst, 'dest')
            fileutils.copytree(src, dest_dir)

            dest_file1 = join(dest_dir, 'a.bat')
            assert os.path.exists(dest_file1)
            assert filetype.is_readable(dest_file1)

            dest_file2 = join(dest_dir, 'subtxt', 'a.txt')
            assert os.path.exists(dest_file2)
            assert filetype.is_readable(dest_file2)

        finally:
            fileutils.chmod(src, fileutils.RW, recurse=True)
            fileutils.chmod(dst, fileutils.RW, recurse=True)


class TestFileUtils(FileBasedTesting):
    test_data_dir = os.path.join(os.path.dirname(__file__), 'data')

    @skipIf(on_windows, 'Windows handles special files differently.')
    def test_copytree_does_not_copy_fifo(self):
        # Windows does not support pipes
        src = self.get_test_loc('fileutils/filetype', copy=True)
        dest = self.get_temp_dir()
        src_file = join(src, 'myfifo')
        os.mkfifo(src_file)  # NOQA
        dest_dir = join(dest, 'dest')
        fileutils.copytree(src, dest_dir)
        assert not os.path.exists(join(dest_dir, 'myfifo'))

    def test_copyfile_keeps_modified_date(self):
        test_file = self.get_test_loc('fileutils/exec/subtxt/a.txt', copy=True)
        dest = self.get_temp_file()
        expected = 1289918700
        os.utime(test_file, (expected, expected))
        fileutils.copyfile(test_file, dest)
        result = os.stat(dest).st_mtime
        assert result == expected

    def test_copyfile_can_copy_file_to_dir_keeping_full_file_name(self):
        test_file = self.get_test_loc('fileutils/exec/subtxt/a.txt', copy=True)
        dest = self.get_temp_dir()
        expected = os.path.join(dest, 'a.txt')
        fileutils.copyfile(test_file, dest)
        assert os.path.exists(expected)

    def test_resource_name(self):
        assert fileutils.resource_name('/a/b/d/f/f') == 'f'
        assert fileutils.resource_name('/a/b/d/f/f/') == 'f'
        assert fileutils.resource_name('a/b/d/f/f/') == 'f'
        assert fileutils.resource_name('/a/b/d/f/f.a') == 'f.a'
        assert fileutils.resource_name('/a/b/d/f/f.a/') == 'f.a'
        assert fileutils.resource_name('a/b/d/f/f.a') == 'f.a'
        assert fileutils.resource_name('f.a') == 'f.a'

    @skipIf(on_windows, 'Windows FS encoding is ... different!')
    def test_fsdecode_and_fsencode_are_idempotent(self):
        a = b'foo\xb1bar'
        b = u'foo\udcb1bar'
        assert os.fsencode(os.fsdecode(a)) == a
        assert os.fsencode(os.fsdecode(b)) == a
        assert os.fsdecode(os.fsencode(a)) == b
        assert os.fsdecode(os.fsencode(b)) == b


class TestFileUtilsWalk(FileBasedTesting):
    test_data_dir = os.path.join(os.path.dirname(__file__), 'data')

    def test_os_walk_with_unicode_path(self):
        test_dir = self.extract_test_zip('fileutils/walk/unicode.zip')
        test_dir = join(test_dir, 'unicode')

        test_dir = str(test_dir)
        result = list(os.walk(test_dir))
        expected = [
            (str(test_dir), ['a'], [u'2.csv']),
            (str(test_dir) + sep + 'a', [], [u'gru\u0308n.png'])
        ]
        assert result == expected

    def test_fileutils_walk(self):
        test_dir = self.get_test_loc('fileutils/walk')
        base = self.get_test_loc('fileutils')
        result = [(as_posixpath(t.replace(base, '')), d, sorted(f),) for t, d, f in fileutils.walk(test_dir)]
        expected = [
            ('/walk', ['d1'], ['f', 'unicode.zip']),
            ('/walk/d1', ['d2'], ['f1']),
            ('/walk/d1/d2', ['d3'], ['f2']),
            ('/walk/d1/d2/d3', [], ['f3'])
        ]
        assert result == expected

    def test_fileutils_walk_with_unicode_path(self):
        test_dir = self.extract_test_zip('fileutils/walk/unicode.zip')
        test_dir = join(test_dir, 'unicode')

        result = list(x[-1] for x in fileutils.walk(test_dir))
        expected = [[u'2.csv'], [u'gru\u0308n.png']]
        assert result == expected

    def test_fileutils_walk_can_walk_a_single_file(self):
        test_file = self.get_test_loc('fileutils/walk/f')
        result = list(fileutils.walk(test_file))
        expected = [
            (fileutils.parent_directory(test_file), [], ['f'])
        ]
        assert result == expected

    def test_fileutils_walk_can_walk_an_empty_dir(self):
        test_dir = self.get_temp_dir()
        result = list(fileutils.walk(test_dir))
        expected = [
            (test_dir, [], [])
        ]
        assert result == expected

    @skipIf(on_macos_14_or_higher, 'Cannot handle yet byte paths on macOS 10.14+. See https://github.com/nexB/scancode-toolkit/issues/1635')
    def test_walk_can_walk_non_utf8_path_from_unicode_path(self):
        test_dir = self.extract_test_tar_raw('fileutils/walk_non_utf8/non_unicode.tgz')
        test_dir = join(test_dir, 'non_unicode')

        if not on_linux:
            test_dir = str(test_dir)
        result = list(fileutils.walk(test_dir))[0]
        _dirpath, _dirnames, filenames = result
        assert len(filenames) == 18

    @skipIf(on_macos_14_or_higher, 'Cannot handle yet byte paths on macOS 10.14+. See https://github.com/nexB/scancode-toolkit/issues/1635')
    def test_os_walk_can_walk_non_utf8_path_from_unicode_path(self):
        test_dir = self.extract_test_tar_raw('fileutils/walk_non_utf8/non_unicode.tgz')
        test_dir = join(test_dir, 'non_unicode')

        result = list(os.walk(test_dir))[0]
        _dirpath, _dirnames, filenames = result
        assert len(filenames) == 18

    @skipIf(on_windows, 'os.symlink does not work on Windows')
    def test_walk_on_symlinks(self):
        test_dir = self.get_test_loc('symlink/walk', copy=True)
        temp_dir = fileutils.get_temp_dir()
        test_link = join(temp_dir, 'test-dir-link')
        os.symlink(test_dir, test_link)
        results = list(fileutils.walk(test_link, follow_symlinks=True))
        results = [(os.path.basename(top), dirs, files) for top, dirs, files in results]
        expected = [
            ('test-dir-link', ['dir'], ['a']),
            ('dir', [], ['b'])
        ]
        assert results == expected


class TestFileUtilsIter(FileBasedTesting):
    test_data_dir = os.path.join(os.path.dirname(__file__), 'data')

    def test_resource_iter(self):
        test_dir = self.get_test_loc('fileutils/walk')
        base = self.get_test_loc('fileutils')
        result = [as_posixpath(f.replace(base, '')) for f in fileutils.resource_iter(test_dir, with_dirs=False)]
        expected = [
            '/walk/f',
            '/walk/unicode.zip',
            '/walk/d1/f1',
            '/walk/d1/d2/f2',
            '/walk/d1/d2/d3/f3'
        ]
        assert sorted(result) == sorted(expected)

    def test_resource_iter_can_iterate_a_single_file(self):
        test_file = self.get_test_loc('fileutils/walk/f')
        result = [as_posixpath(f) for f in fileutils.resource_iter(test_file, with_dirs=False)]
        expected = [as_posixpath(test_file)]
        assert result == expected

    def test_resource_iter_can_iterate_a_single_file_with_dirs(self):
        test_file = self.get_test_loc('fileutils/walk/f')
        result = [as_posixpath(f) for f in fileutils.resource_iter(test_file, with_dirs=True)]
        expected = [as_posixpath(test_file)]
        assert result == expected

    def test_resource_iter_can_walk_an_empty_dir(self):
        test_dir = self.get_temp_dir()
        result = list(fileutils.resource_iter(test_dir, with_dirs=False))
        expected = []
        assert result == expected

    def test_resource_iter_can_walk_an_empty_dir_with_dirs(self):
        test_dir = self.get_temp_dir()
        result = list(fileutils.resource_iter(test_dir, with_dirs=False))
        expected = []
        assert result == expected

    def test_resource_iter_without_dir(self):
        test_dir = self.get_test_loc('fileutils/walk')
        base = self.get_test_loc('fileutils')
        result = sorted([as_posixpath(f.replace(base, ''))
                         for f in fileutils.resource_iter(test_dir, with_dirs=False)])
        expected = [
            '/walk/f',
            '/walk/unicode.zip',
            '/walk/d1/f1',
            '/walk/d1/d2/f2',
            '/walk/d1/d2/d3/f3'
        ]
        assert sorted(result) == sorted(expected)

    def test_resource_iter_with_dirs(self):
        test_dir = self.get_test_loc('fileutils/walk')
        base = self.get_test_loc('fileutils')
        result = sorted([as_posixpath(f.replace(base, ''))
                         for f in fileutils.resource_iter(test_dir, with_dirs=True)])
        expected = [
            '/walk/d1',
            '/walk/d1/d2',
            '/walk/d1/d2/d3',
            '/walk/d1/d2/d3/f3',
            '/walk/d1/d2/f2',
            '/walk/d1/f1',
            '/walk/f',
            '/walk/unicode.zip'
        ]
        assert sorted(result) == sorted(expected)

    def test_resource_iter_return_byte_on_byte_input(self):
        test_dir = self.get_test_loc('fileutils/walk')
        base = self.get_test_loc('fileutils')
        result = sorted([as_posixpath(f.replace(base, ''))
                         for f in fileutils.resource_iter(test_dir, with_dirs=True)])
        expected = [
            '/walk/d1',
            '/walk/d1/d2',
            '/walk/d1/d2/d3',
            '/walk/d1/d2/d3/f3',
            '/walk/d1/d2/f2',
            '/walk/d1/f1',
            '/walk/f',
            '/walk/unicode.zip'
        ]
        assert sorted(result) == sorted(expected)
        assert all(isinstance(p, str) for p in result)

    def test_resource_iter_return_unicode_on_unicode_input(self):
        test_dir = self.get_test_loc('fileutils/walk')
        base = str(self.get_test_loc('fileutils'))
        result = sorted([as_posixpath(f.replace(base, ''))
                         for f in fileutils.resource_iter(test_dir, with_dirs=True)])
        expected = [
            u'/walk/d1',
            u'/walk/d1/d2',
            u'/walk/d1/d2/d3',
            u'/walk/d1/d2/d3/f3',
            u'/walk/d1/d2/f2',
            u'/walk/d1/f1',
            u'/walk/f',
            u'/walk/unicode.zip'
        ]
        assert sorted(result) == sorted(expected)
        assert all(isinstance(p, str) for p in result)

    def test_resource_iter_can_walk_unicode_path_with_zip(self):
        test_dir = self.extract_test_zip('fileutils/walk/unicode.zip')
        test_dir = join(test_dir, 'unicode')

        test_dir = str(test_dir)

        result = sorted([p.replace(test_dir, '') for p in fileutils.resource_iter(test_dir)])
        if on_linux:
            expected = [
                u'/2.csv',
                u'/a',
                u'/a/gru\u0308n.png'
            ]
        elif on_mac:
            expected = [
                u'/2.csv',
                u'/a',
                u'/a/gru\u0308n.png'
            ]
        elif on_windows:
            expected = [
                u'\\2.csv',
                u'\\a',
                u'\\a\\gru\u0308n.png'
            ]
        assert result == expected

    @skipIf(on_macos_14_or_higher, 'Cannot handle yet byte paths on macOS 10.14+. See https://github.com/nexB/scancode-toolkit/issues/1635')
    def test_resource_iter_can_walk_non_utf8_path_from_unicode_path_with_dirs(self):
        test_dir = self.extract_test_tar_raw('fileutils/walk_non_utf8/non_unicode.tgz')
        test_dir = join(test_dir, 'non_unicode')

        result = list(fileutils.resource_iter(test_dir, with_dirs=True))
        assert len(result) == 18

    @skipIf(on_macos_14_or_higher, 'Cannot handle yet byte paths on macOS 10.14+. See https://github.com/nexB/scancode-toolkit/issues/1635')
    def test_resource_iter_can_walk_non_utf8_path_from_unicode_path(self):
        test_dir = self.extract_test_tar_raw('fileutils/walk_non_utf8/non_unicode.tgz')
        test_dir = join(test_dir, 'non_unicode')

        result = list(fileutils.resource_iter(test_dir, with_dirs=False))
        assert len(result) == 18

    @skipIf(on_windows, 'Symlinks do not work well on Windows')
    def test_resource_iter_follow_symlinks(self):
        test_dir = self.get_test_loc('symlink/walk', copy=True)
        temp_dir = fileutils.get_temp_dir()
        test_link = join(temp_dir, 'test-dir-link')
        os.symlink(test_dir, test_link)
        result = [os.path.basename(f) for f in fileutils.resource_iter(test_dir, follow_symlinks=True)]
        expected = [
            'dir',
            'a',
            'b'
        ]
        assert sorted(result) == sorted(expected)


class TestBaseName(FileBasedTesting):
    test_data_dir = os.path.join(os.path.dirname(__file__), 'data')

    def test_file_base_name_on_path_and_location_1(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'a/.a/file'
        expected_name = 'file'
        result = fileutils.file_base_name(test_file)
        assert result == expected_name
        result = fileutils.file_base_name(join(test_dir, test_file))
        assert result == expected_name

    def test_file_base_name_on_file_path_for_dot_file  (self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'a/.a/'
        expected_name = '.a'
        result = fileutils.file_base_name(test_file)
        assert result == expected_name
        result = fileutils.file_base_name(join(test_dir, test_file))
        assert result == expected_name

    def test_file_base_name_on_file_path_for_dot_file_with_extension(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'a/b/.a.b'
        expected_name = '.a'
        result = fileutils.file_base_name(test_file)
        assert result == expected_name
        result = fileutils.file_base_name(join(test_dir, test_file))
        assert result == expected_name

    def test_file_base_name_on_file_path_for_file_with_unknown_composed_extension(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'a/b/a.tag.gz'
        expected_name = 'a.tag'
        result = fileutils.file_base_name(test_file)
        assert result == expected_name
        result = fileutils.file_base_name(join(test_dir, test_file))
        assert result == expected_name

    def test_file_base_name_on_file_path_for_file_with_known_composed_extension(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'a/b/a.tar.gz'
        expected_name = 'a'
        result = fileutils.file_base_name(test_file)
        assert result == expected_name
        result = fileutils.file_base_name(join(test_dir, test_file))
        assert result == expected_name

    def test_file_base_name_on_dir_path(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'a/b/'
        expected_name = 'b'
        result = fileutils.file_base_name(test_file)
        assert result == expected_name
        result = fileutils.file_base_name(join(test_dir, test_file))
        assert result == expected_name

    def test_file_base_name_on_plain_file(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'a/f.a'
        expected_name = 'f'
        result = fileutils.file_base_name(test_file)
        assert result == expected_name
        result = fileutils.file_base_name(join(test_dir, test_file))
        assert result == expected_name

    def test_file_base_name_on_plain_file_with_parent_dir_extension(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'f.a/a.c'
        expected_name = 'a'
        result = fileutils.file_base_name(test_file)
        assert result == expected_name
        result = fileutils.file_base_name(join(test_dir, test_file))
        assert result == expected_name

    def test_file_base_name_on_path_for_plain_dir(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'a/'
        expected_name = 'a'
        result = fileutils.file_base_name(test_file)
        assert result == expected_name
        result = fileutils.file_base_name(join(test_dir, test_file))
        assert result == expected_name

    def test_file_base_name_on_path_for_plain_dir_with_extension(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'f.a/'
        expected_name = 'f.a'
        result = fileutils.file_base_name(test_file)
        assert result == expected_name
        result = fileutils.file_base_name(join(test_dir, test_file))
        assert result == expected_name

    def test_file_base_name_on_path_for_plain_file(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'tst'
        expected_name = 'tst'
        result = fileutils.file_base_name(test_file)
        assert result == expected_name
        result = fileutils.file_base_name(join(test_dir, test_file))
        assert result == expected_name


class TestFileName(FileBasedTesting):
    test_data_dir = os.path.join(os.path.dirname(__file__), 'data')

    def test_file_name_on_path_and_location_1(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'a/.a/file'
        expected_name = 'file'
        result = fileutils.file_name(test_file)
        assert result == expected_name
        result = fileutils.file_name((os.path.join(test_dir, test_file)))
        assert result == expected_name

    def test_file_name_on_path_and_location_2(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'a/.a/'
        expected_name = '.a'
        result = fileutils.file_name(test_file)
        assert result == expected_name
        result = fileutils.file_name((os.path.join(test_dir, test_file)))
        assert result == expected_name

    def test_file_name_on_path_and_location_3(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'a/b/.a.b'
        expected_name = '.a.b'
        result = fileutils.file_name(test_file)
        assert result == expected_name
        result = fileutils.file_name((os.path.join(test_dir, test_file)))
        assert result == expected_name

    def test_file_name_on_path_and_location_4(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'a/b/a.tag.gz'
        expected_name = 'a.tag.gz'
        result = fileutils.file_name(test_file)
        assert result == expected_name
        result = fileutils.file_name((os.path.join(test_dir, test_file)))
        assert result == expected_name

    def test_file_name_on_path_and_location_5(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'a/b/'
        expected_name = 'b'
        result = fileutils.file_name(test_file)
        assert result == expected_name
        result = fileutils.file_name((os.path.join(test_dir, test_file)))
        assert result == expected_name

    def test_file_name_on_path_and_location_6(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'a/f.a'
        expected_name = 'f.a'
        result = fileutils.file_name(test_file)
        assert result == expected_name
        result = fileutils.file_name((os.path.join(test_dir, test_file)))
        assert result == expected_name

    def test_file_name_on_path_and_location_7(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'a/'
        expected_name = 'a'
        result = fileutils.file_name(test_file)
        assert result == expected_name
        result = fileutils.file_name((os.path.join(test_dir, test_file)))
        assert result == expected_name

    def test_file_name_on_path_and_location_8(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'f.a/a.c'
        expected_name = 'a.c'
        result = fileutils.file_name(test_file)
        assert result == expected_name
        result = fileutils.file_name((os.path.join(test_dir, test_file)))
        assert result == expected_name

    def test_file_name_on_path_and_location_9(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'f.a/'
        expected_name = 'f.a'
        result = fileutils.file_name(test_file)
        assert result == expected_name
        result = fileutils.file_name((os.path.join(test_dir, test_file)))
        assert result == expected_name

    def test_file_name_on_path_and_location_10(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'tst'
        expected_name = 'tst'
        result = fileutils.file_name(test_file)
        assert result == expected_name
        result = fileutils.file_name((os.path.join(test_dir, test_file)))
        assert result == expected_name


class TestFileExtension(FileBasedTesting):
    test_data_dir = os.path.join(os.path.dirname(__file__), 'data')

    def test_file_extension_on_path_and_location_1(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'a/.a/file'
        expected_name = ''
        result = fileutils.file_extension(test_file)
        assert result == expected_name
        result = fileutils.file_extension((os.path.join(test_dir, test_file)))
        assert result == expected_name

    def test_file_extension_on_path_and_location_2(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'a/.a/'
        expected_name = ''
        result = fileutils.file_extension(test_file)
        assert result == expected_name
        result = fileutils.file_extension((os.path.join(test_dir, test_file)))
        assert result == expected_name

    def test_file_extension_on_path_and_location_3(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'a/b/.a.b'
        expected_name = '.b'
        result = fileutils.file_extension(test_file)
        assert result == expected_name
        result = fileutils.file_extension((os.path.join(test_dir, test_file)))
        assert result == expected_name

    def test_file_extension_on_path_and_location_4(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'a/b/a.tag.gz'
        expected_name = '.gz'
        result = fileutils.file_extension(test_file)
        assert result == expected_name
        result = fileutils.file_extension((os.path.join(test_dir, test_file)))
        assert result == expected_name

    def test_file_extension_on_path_and_location_5(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'a/b/'
        expected_name = ''
        result = fileutils.file_extension(test_file)
        assert result == expected_name
        result = fileutils.file_extension((os.path.join(test_dir, test_file)))
        assert result == expected_name

    def test_file_extension_on_path_and_location_6(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'a/f.a'
        expected_name = '.a'
        result = fileutils.file_extension(test_file)
        assert result == expected_name
        result = fileutils.file_extension((os.path.join(test_dir, test_file)))
        assert result == expected_name

    def test_file_extension_on_path_and_location_7(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'a/'
        expected_name = ''
        result = fileutils.file_extension(test_file)
        assert result == expected_name
        result = fileutils.file_extension((os.path.join(test_dir, test_file)))
        assert result == expected_name

    def test_file_extension_on_path_and_location_8(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'f.a/a.c'
        expected_name = '.c'
        result = fileutils.file_extension(test_file)
        assert result == expected_name
        result = fileutils.file_extension((os.path.join(test_dir, test_file)))
        assert result == expected_name

    def test_file_extension_on_path_and_location_9(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'f.a/'
        expected_name = ''
        result = fileutils.file_extension(test_file)
        assert result == expected_name
        result = fileutils.file_extension((os.path.join(test_dir, test_file)))
        assert result == expected_name

    def test_file_extension_on_path_and_location_10(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'tst'
        expected_name = ''
        result = fileutils.file_extension(test_file)
        assert result == expected_name
        result = fileutils.file_extension((os.path.join(test_dir, test_file)))
        assert result == expected_name

    def test_splitext_base(self):
        expected = 'path', '.ext'
        assert fileutils.splitext('C:\\dir\\path.ext') == expected

    def test_splitext_directories_even_with_dotted_names_have_no_extension(self):
        import ntpath
        expected = 'path.ext', ''
        assert fileutils.splitext('C:\\dir\\path.ext' + ntpath.sep) == expected

        expected = 'path.ext', ''
        assert fileutils.splitext('/dir/path.ext/') == expected

        expected = 'file', '.txt'
        assert fileutils.splitext('/some/file.txt') == expected

    def test_splitext_composite_extensions_for_tarballs_are_properly_handled(self):
        expected = 'archive', '.tar.gz'
        assert fileutils.splitext('archive.tar.gz') == expected

    def test_splitext_name_base(self):
        expected = 'path', '.ext'
        assert fileutils.splitext_name('path.ext') == expected

    def test_splitext_name_directories_have_no_extension(self):
        expected = 'path.ext', ''
        assert fileutils.splitext_name('path.ext', is_file=False) == expected

        expected = 'file', '.txt'
        assert fileutils.splitext_name('file.txt') == expected

    def test_splitext_name_composite_extensions_for_tarballs_are_properly_handled(self):
        expected = 'archive', '.tar.gz'
        assert fileutils.splitext_name('archive.tar.gz') == expected

    def test_splitext_name_dotfile_are_properly_handled(self):
        expected = '.dotfile', ''
        assert fileutils.splitext_name('.dotfile') == expected
        expected = '.dotfile', '.this'
        assert fileutils.splitext_name('.dotfile.this') == expected


class TestParentDir(FileBasedTesting):
    test_data_dir = os.path.join(os.path.dirname(__file__), 'data')

    def test_parent_directory_on_path_and_location_1(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'a/.a/file'
        expected_name = 'a/.a/'
        result = fileutils.parent_directory(test_file)
        result = fileutils.as_posixpath(result)
        assert result == expected_name

        result = fileutils.parent_directory((os.path.join(test_dir, test_file)))
        result = fileutils.as_posixpath(result)
        assert result.endswith(expected_name)

    def test_parent_directory_on_path_and_location_2(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'a/.a/'
        expected_name = 'a/'
        result = fileutils.parent_directory(test_file)
        result = fileutils.as_posixpath(result)
        assert result == expected_name

        result = fileutils.parent_directory((os.path.join(test_dir, test_file)))
        result = fileutils.as_posixpath(result)
        assert result.endswith(expected_name)

    def test_parent_directory_on_path_and_location_3(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'a/b/.a.b'
        expected_name = 'a/b/'
        result = fileutils.parent_directory(test_file)
        result = fileutils.as_posixpath(result)
        assert result == expected_name

        result = fileutils.parent_directory((os.path.join(test_dir, test_file)))
        result = fileutils.as_posixpath(result)
        assert result.endswith(expected_name)

    def test_parent_directory_on_path_and_location_4(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'a/b/a.tag.gz'
        expected_name = 'a/b/'
        result = fileutils.parent_directory(test_file)
        result = fileutils.as_posixpath(result)
        assert result == expected_name

        result = fileutils.parent_directory((os.path.join(test_dir, test_file)))
        result = fileutils.as_posixpath(result)
        assert result.endswith(expected_name)

    def test_parent_directory_on_path_and_location_5(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'a/b/'
        expected_name = 'a/'
        result = fileutils.parent_directory(test_file)
        result = fileutils.as_posixpath(result)
        assert result == expected_name

        result = fileutils.parent_directory((os.path.join(test_dir, test_file)))
        result = fileutils.as_posixpath(result)
        assert result.endswith(expected_name)

    def test_parent_directory_on_path_and_location_6(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'a/f.a'
        expected_name = 'a/'
        result = fileutils.parent_directory(test_file)
        result = fileutils.as_posixpath(result)
        assert result == expected_name

        result = fileutils.parent_directory((os.path.join(test_dir, test_file)))
        result = fileutils.as_posixpath(result)
        assert result.endswith(expected_name)

    def test_parent_directory_on_path_and_location_7(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'a/'
        expected_name = '/'
        result = fileutils.parent_directory(test_file)
        result = fileutils.as_posixpath(result)
        assert result == expected_name

        result = fileutils.parent_directory((os.path.join(test_dir, test_file)))
        result = fileutils.as_posixpath(result)
        assert result.endswith(expected_name)

    def test_parent_directory_on_path_and_location_8(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'f.a/a.c'
        expected_name = 'f.a/'
        result = fileutils.parent_directory(test_file)
        result = fileutils.as_posixpath(result)
        assert result == expected_name

        result = fileutils.parent_directory((os.path.join(test_dir, test_file)))
        result = fileutils.as_posixpath(result)
        assert result.endswith(expected_name)

    def test_parent_directory_on_path_and_location_9(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'f.a/'
        expected_name = '/'
        result = fileutils.parent_directory(test_file)
        result = fileutils.as_posixpath(result)
        assert result == expected_name

        result = fileutils.parent_directory((os.path.join(test_dir, test_file)))
        result = fileutils.as_posixpath(result)
        assert result.endswith(expected_name)

    def test_parent_directory_on_path_and_location_10(self):
        test_dir = self.get_test_loc('fileutils/basename')
        test_file = 'tst'
        expected_name = '/'
        result = fileutils.parent_directory(test_file)
        result = fileutils.as_posixpath(result)
        assert result == expected_name

        result = fileutils.parent_directory((os.path.join(test_dir, test_file)))
        result = fileutils.as_posixpath(result)
        assert result.endswith(expected_name)



# -*- coding: utf-8 -*-
#
# Copyright (c) nexB Inc. and others. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# See http://www.apache.org/licenses/LICENSE-2.0 for the license text.
# See https://github.com/nexB/commoncode for support or download.
# See https://aboutcode.org for more information about nexB OSS projects.
#

import os
import sys

from unittest.case import skipIf

from commoncode import command
from commoncode.testcase import FileBasedTesting
from commoncode.system import on_linux
from commoncode.system import on_mac
from commoncode.system import on_windows
from commoncode.system import py36


class TestCommand(FileBasedTesting):
    test_data_dir = os.path.join(os.path.dirname(__file__), 'data')

    @skipIf(py36, 'This fails on Python 3.6 https://bugs.python.org/issue26919')
    def test_execute_can_handle_non_ascii_output(self):
        # Popen returns a *binary* string with non-ascii chars: skips these
        python = sys.executable
        rc, stdout, stderr = command.execute(
            python, ['-c', 'print("non ascii: été just passed it !")']
        )
        assert stderr == ''
        assert stdout == 'non ascii: ete just passed it !'
        assert rc == 0
        # do not throw exception
        stdout.encode('ascii')

    def test_execute_(self):
        python = sys.executable
        rc, stdout, stderr = command.execute(
            python, ['-c', 'print("foobar")']
        )
        assert stderr == ''
        assert stdout == 'foobar'
        assert rc == 0
        # do not throw exception
        stdout.encode('ascii')

    def test_execute2(self):
        python = sys.executable
        rc, stdout, stderr = command.execute2(
            python, ['-c', 'print("foobar")']
        )
        assert stderr == ''
        assert stdout == 'foobar'
        assert rc == 0
        # do not throw exception
        stdout.encode('ascii')

    @skipIf(not on_linux, 'Linux only')
    def test_update_path_var_on_linux(self):
        existing_path_var = '/usr/bin:/usr/local'

        new_path = b'foo\xb1bar'
        updated_path = command.update_path_var(existing_path_var, new_path)
        assert updated_path == 'foo\udcb1bar:/usr/bin:/usr/local'

        new_path = u'/bin/foo\udcb1bar'
        updated_path = command.update_path_var(updated_path, new_path)
        assert updated_path == '/bin/foo\udcb1bar:foo\udcb1bar:/usr/bin:/usr/local'

        new_path = b'/bin/foo\xb1bar'
        updated_path = command.update_path_var(updated_path, new_path)
        assert updated_path == '/bin/foo\udcb1bar:foo\udcb1bar:/usr/bin:/usr/local'

        new_path = u'foo\udcb1bar'
        updated_path = command.update_path_var(updated_path, new_path)
        assert updated_path == '/bin/foo\udcb1bar:foo\udcb1bar:/usr/bin:/usr/local'

        new_path = b'foo\xb1bar'
        updated_path = command.update_path_var(updated_path, new_path)
        assert updated_path == '/bin/foo\udcb1bar:foo\udcb1bar:/usr/bin:/usr/local'

    @skipIf(not on_mac, 'Mac only')
    def test_update_path_var_on_mac(self):
        existing_path_var = '/usr/bin:/usr/local'

        new_path = u'foo\udcb1bar'
        updated_path = command.update_path_var(existing_path_var, new_path)
        assert updated_path == 'foo\udcb1bar:/usr/bin:/usr/local'

        new_path = b'/bin/foo\xb1bar'
        updated_path = command.update_path_var(updated_path, new_path)
        assert updated_path == '/bin/foo\udcb1bar:foo\udcb1bar:/usr/bin:/usr/local'

        new_path = u'foo\udcb1bar'
        updated_path = command.update_path_var(updated_path, new_path)
        assert updated_path == '/bin/foo\udcb1bar:foo\udcb1bar:/usr/bin:/usr/local'

    @skipIf(not on_windows, 'Windows only')
    def test_update_path_var_on_windows(self):
        existing_path_var = u'c:\\windows;C:Program Files'

        new_path = u'foo\udcb1bar'
        updated_path = command.update_path_var(existing_path_var, new_path)
        assert updated_path == u'foo\udcb1bar;c:\\windows;C:Program Files'

        new_path = u'foo\udcb1bar'
        updated_path = command.update_path_var(updated_path, new_path)
        assert updated_path == u'foo\udcb1bar;c:\\windows;C:Program Files'

    def test_searchable_paths(self):
        d1 = self.get_temp_dir('foo')
        d2 = self.get_temp_dir('bar')
        ps = os.pathsep
        os.environ['FOO_SCANCODE_TEST1'] = f'{ps}{d1}{ps}{ps}{d2}{ps}'
        os.environ['FOO_SCANCODE_TEST2'] = f'{ps}{d2}{ps}{ps}{d1}{ps}/NOTADIR'

        env_vars = 'FOO_SCANCODE_TEST1', 'FOO_SCANCODE_TEST2'
        expected = d1, d2, d2, d1

        results = command.searchable_paths(env_vars=env_vars)
        if on_windows:
            for res, exp in zip(results, expected):
                _, _, r = res.rpartition('\\')
                _, _, e = exp.rpartition('\\')
                assert r == e

        elif on_mac:
            # macOS somehow adds a /private to the paths in the CI as a side-
            # effect of calling "realpath" and likely resolving links
            expected = f'/private{d1}', f'/private{d2}', f'/private{d2}', f'/private{d1}'
            assert expected == results
        else:
            assert expected == results

    def test_find_in_path(self):
        d1 = self.get_temp_dir('foo')
        d2 = self.get_temp_dir('bar')
        filename = 'baz'

        assert None == command.find_in_path(filename, searchable_paths=(d1, d2,))

        f2 = os.path.join(d2, filename)
        with open(f2, 'w') as o:
            o.write('some')

        assert f2 == command.find_in_path(filename, searchable_paths=(d1, d2,))
        assert f2 == command.find_in_path(filename, searchable_paths=(d2, d1,))

        f1 = os.path.join(d1, filename)
        with open(f1, 'w') as o:
            o.write('some')

        assert f1 == command.find_in_path(filename, searchable_paths=(d1, d2,))
        assert f2 == command.find_in_path(filename, searchable_paths=(d2, d1,))




# Copyright (c) 2003-2012 Raymond Hettinger
# SPDX-License-Identifier: Python-2.0


def sparsify(d):
    """
    http://code.activestate.com/recipes/198157-improve-dictionary-lookup-performance/
    Created by Raymond Hettinger on Sun, 4 May 2003
    Reduce average dictionary lookup time by making the internal tables more sparse.
    Improve dictionary sparsity.

    The dict.update() method makes space for non-overlapping keys.
    Giving it a dictionary with 100% overlap will build the same
    dictionary in the larger space.  The resulting dictionary will
    be no more that 1/3 full.  As a result, lookups require less
    than 1.5 probes on average.

    Example:
    >>> sparsify({1: 3, 4: 5})
    {1: 3, 4: 5}
    """
    e = d.copy()
    d.update(e)
    return d



#
# Copyright (c) nexB Inc. and others. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# See http://www.apache.org/licenses/LICENSE-2.0 for the license text.
# See https://github.com/nexB/commoncode for support or download.
# See https://aboutcode.org for more information about nexB OSS projects.
#

from unittest import TestCase

from commoncode import paths


class TestPortablePath(TestCase):

    def test_safe_path_mixed_slashes(self):
        test = paths.safe_path('C:\\Documents and Settings\\Boki\\Desktop\\head\\patches\\drupal6/drupal.js')
        expected = 'C/Documents_and_Settings/Boki/Desktop/head/patches/drupal6/drupal.js'
        assert test == expected

    def test_safe_path_mixed_slashes_and_spaces(self):
        test = paths.safe_path('C:\\Documents and Settings\\Boki\\Desktop\\head\\patches\\parallel uploads/drupal.js')
        expected = 'C/Documents_and_Settings/Boki/Desktop/head/patches/parallel_uploads/drupal.js'
        assert test == expected

    def test_safe_path_windows_style(self):
        test = paths.safe_path('C:\\Documents and Settings\\Administrator\\Desktop\\siftDemoV4_old\\defs.h')
        expected = 'C/Documents_and_Settings/Administrator/Desktop/siftDemoV4_old/defs.h'
        assert test == expected

    def test_safe_path_windows_style_mixed_slashes_no_spaces(self):
        test = paths.safe_path('C:\\Documents and Settings\\Boki\\Desktop\\head\\patches\\imagefield/imagefield.css')
        expected = 'C/Documents_and_Settings/Boki/Desktop/head/patches/imagefield/imagefield.css'
        assert test == expected

    def test_safe_path_windows_style_spaces(self):
        test = paths.safe_path('C:\\Documents and Settings\\Boki\\Desktop\\head\\patches\\js delete\\imagefield.css')
        expected = 'C/Documents_and_Settings/Boki/Desktop/head/patches/js_delete/imagefield.css'
        assert test == expected

    def test_safe_path_windows_style_posix_slashes(self):
        test = paths.safe_path('C:/Documents and Settings/Alex Burgel/workspace/Hibernate3.2/test/org/hibernate/test/AllTests.java')
        expected = 'C/Documents_and_Settings/Alex_Burgel/workspace/Hibernate3.2/test/org/hibernate/test/AllTests.java'
        assert test == expected

    def test_safe_path_windows_style_relative(self):
        test = paths.safe_path('includes\\webform.components.inc')
        expected = 'includes/webform.components.inc'
        assert test == expected

    def test_safe_path_windows_style_absolute_trailing_slash(self):
        test = paths.safe_path('\\includes\\webform.components.inc\\')
        expected = 'includes/webform.components.inc'
        assert test == expected

    def test_safe_path_posix_style_relative(self):
        test = paths.safe_path('includes/webform.components.inc')
        expected = 'includes/webform.components.inc'
        assert test == expected

    def test_safe_path_posix_style_absolute_trailing_slash(self):
        test = paths.safe_path('/includes/webform.components.inc/')
        expected = 'includes/webform.components.inc'
        assert test == expected

    def test_safe_path_posix_style_french_char(self):
        test = paths.safe_path('/includes/webform.compon\xc3nts.inc/')
        expected = 'includes/webform.componAnts.inc'
        assert test == expected

    def test_safe_path_posix_style_chinese_char(self):
        test = paths.safe_path(b'/includes/webform.compon\xd2\xaants.inc/')
        expected = 'includes/webform.componNSnts.inc'
        assert test == expected

    def test_safe_path_windows_style_dots(self):
        test = paths.safe_path('\\includes\\..\\webform.components.inc\\')
        expected = 'webform.components.inc'
        assert test == expected

    def test_safe_path_windows_style_many_dots(self):
        test = paths.safe_path('.\\includes\\.\\..\\..\\..\\webform.components.inc\\.')
        expected = 'dotdot/dotdot/webform.components.inc'
        assert test == expected

    def test_safe_path_posix_style_dots(self):
        test = paths.safe_path('includes/../webform.components.inc')
        expected = 'webform.components.inc'
        assert test == expected

    def test_safe_path_posix_style_many_dots(self):
        test = paths.safe_path('./includes/./../../../../webform.components.inc/.')
        expected = 'dotdot/dotdot/dotdot/webform.components.inc'
        assert test == expected

    def test_resolve_mixed_slash(self):
        test = paths.resolve('C:\\..\\./drupal.js')
        expected = 'C/drupal.js'
        assert test == expected

    def test_resolve_2(self):
        test = paths.resolve('\\includes\\..\\webform.components.inc\\')
        expected = 'webform.components.inc'
        assert test == expected

    def test_resolve_3(self):
        test = paths.resolve('includes/../webform.components.inc')
        expected = 'webform.components.inc'
        assert test == expected

    def test_resolve_4(self):
        test = paths.resolve('////.//includes/./../..//..///../webform.components.inc/.')
        expected = 'dotdot/dotdot/dotdot/webform.components.inc'
        assert test == expected

    def test_resolve_5(self):
        test = paths.resolve(u'////.//includes/./../..//..///../webform.components.inc/.')
        expected = u'dotdot/dotdot/dotdot/webform.components.inc'
        assert test == expected

    def test_resolve_6(self):
        test = paths.resolve('includes/../')
        expected = '.'
        assert test == expected

    def test_portable_filename(self):
        expected = 'A___file__with_Spaces.mov'
        assert paths.portable_filename("A:\\ file/ with Spaces.mov") == expected

        # Test `preserve_spaces` option. Spaces should not be replaced
        expected = 'Program Files (x86)'
        assert paths.portable_filename("Program Files (x86)", preserve_spaces=True) == expected

        # Unresolved relative paths will be treated as a single filename. Use
        # resolve instead if you want to resolve paths:
        expected = '___.._.._etc_passwd'
        assert paths.portable_filename("../../../etc/passwd") == expected

        # Unicode name are transliterated:
        expected = 'This_contain_UMLAUT_umlauts.txt'
        assert paths.portable_filename(u'This contain UMLAUT \xfcml\xe4uts.txt') == expected


class TestCommonPath(TestCase):

    def test_common_path_prefix1(self):
        test = paths.common_path_prefix('/a/b/c', '/a/b/c')
        assert test == ('a/b/c', 3)

    def test_common_path_prefix2(self):
        test = paths.common_path_prefix('/a/b/c', '/a/b')
        assert test == ('a/b', 2)

    def test_common_path_prefix3(self):
        test = paths.common_path_prefix('/a/b', '/a/b/c')
        assert test == ('a/b', 2)

    def test_common_path_prefix4(self):
        test = paths.common_path_prefix('/a', '/a')
        assert test == ('a', 1)

    def test_common_path_prefix_path_root(self):
        test = paths.common_path_prefix('/a/b/c', '/')
        assert test == (None, 0)

    def test_common_path_prefix_root_path(self):
        test = paths.common_path_prefix('/', '/a/b/c')
        assert test == (None, 0)

    def test_common_path_prefix_root_root(self):
        test = paths.common_path_prefix('/', '/')
        assert test == (None, 0)

    def test_common_path_prefix_path_elements_are_similar(self):
        test = paths.common_path_prefix('/a/b/c', '/a/b/d')
        assert test == ('a/b', 2)

    def test_common_path_prefix_no_match(self):
        test = paths.common_path_prefix('/abc/d', '/abe/f')
        assert test == (None, 0)

    def test_common_path_prefix_ignore_training_slashes(self):
        test = paths.common_path_prefix('/a/b/c/', '/a/b/c/')
        assert test == ('a/b/c', 3)

    def test_common_path_prefix8(self):
        test = paths.common_path_prefix('/a/b/c/', '/a/b')
        assert test == ('a/b', 2)

    def test_common_path_prefix10(self):
        test = paths.common_path_prefix('/a/b/c.txt', '/a/b/b.txt')
        assert test == ('a/b', 2)

    def test_common_path_prefix11(self):
        test = paths.common_path_prefix('/a/b/c.txt', '/a/b.txt')
        assert test == ('a', 1)

    def test_common_path_prefix12(self):
        test = paths.common_path_prefix('/a/c/e/x.txt', '/a/d/a.txt')
        assert test == ('a', 1)

    def test_common_path_prefix13(self):
        test = paths.common_path_prefix('/a/c/e/x.txt', '/a/d/')
        assert test == ('a', 1)

    def test_common_path_prefix14(self):
        test = paths.common_path_prefix('/a/c/e/', '/a/d/')
        assert test == ('a', 1)

    def test_common_path_prefix15(self):
        test = paths.common_path_prefix('/a/c/e/', '/a/c/a.txt')
        assert test == ('a/c', 2)

    def test_common_path_prefix16(self):
        test = paths.common_path_prefix('/a/c/e/', '/a/c/f/')
        assert test == ('a/c', 2)

    def test_common_path_prefix17(self):
        test = paths.common_path_prefix('/a/a.txt', '/a/b.txt/')
        assert test == ('a', 1)

    def test_common_path_prefix18(self):
        test = paths.common_path_prefix('/a/c/', '/a/')
        assert test == ('a', 1)

    def test_common_path_prefix19(self):
        test = paths.common_path_prefix('/a/c.txt', '/a/')
        assert test == ('a', 1)

    def test_common_path_prefix20(self):
        test = paths.common_path_prefix('/a/c/', '/a/d/')
        assert test == ('a', 1)

    def test_common_path_suffix(self):
        test = paths.common_path_suffix('/a/b/c', '/a/b/c')
        assert test == ('a/b/c', 3)

    def test_common_path_suffix_absolute_relative(self):
        test = paths.common_path_suffix('a/b/c', '/a/b/c')
        assert test == ('a/b/c', 3)

    def test_common_path_suffix_find_subpath(self):
        test = paths.common_path_suffix('/z/b/c', '/a/b/c')
        assert test == ('b/c', 2)

    def test_common_path_suffix_handles_relative_path(self):
        test = paths.common_path_suffix('a/b', 'a/b')
        assert test == ('a/b', 2)

    def test_common_path_suffix_handles_relative_subpath(self):
        test = paths.common_path_suffix('zsds/adsds/a/b/b/c', 'a//a/d//b/c')
        assert test == ('b/c', 2)

    def test_common_path_suffix_ignore_and_strip_trailing_slash(self):
        test = paths.common_path_suffix('zsds/adsds/a/b/b/c/', 'a//a/d//b/c/')
        assert test == ('b/c', 2)

    def test_common_path_suffix_return_None_if_no_common_suffix(self):
        test = paths.common_path_suffix('/a/b/c', '/')
        assert test == (None, 0)

    def test_common_path_suffix_return_None_if_no_common_suffix2(self):
        test = paths.common_path_suffix('/', '/a/b/c')
        assert test == (None, 0)

    def test_common_path_suffix_match_only_whole_segments(self):
        # only segments are honored, commonality within segment is ignored
        test = paths.common_path_suffix('this/is/aaaa/great/path', 'this/is/aaaaa/great/path')
        assert test == ('great/path', 2)

    def test_common_path_suffix_two_root(self):
        test = paths.common_path_suffix('/', '/')
        assert test == (None, 0)

    def test_common_path_suffix_empty_root(self):
        test = paths.common_path_suffix('', '/')
        assert test == (None, 0)

    def test_common_path_suffix_root_empty(self):
        test = paths.common_path_suffix('/', '')
        assert test == (None, 0)

    def test_common_path_suffix_empty_empty(self):
        test = paths.common_path_suffix('', '')
        assert test == (None, 0)



# -*- coding: utf-8 -*-
#
# Copyright (c) nexB Inc. and others. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# See http://www.apache.org/licenses/LICENSE-2.0 for the license text.
# See https://github.com/nexB/commoncode for support or download.
# See https://aboutcode.org for more information about nexB OSS projects.
#

import logging
import re
import unicodedata

from text_unidecode import unidecode
from bs4.dammit import UnicodeDammit

"""
A text processing module providing functions to process and prepare text
before indexing or fingerprinting such as:
 - case folding
 - conversion of iso latin and unicode to ascii
 - punctuation stripping
 - line separator stripping and conversion
 """

LOG = logging.getLogger(__name__)


def lines(s):
    """
    Split a string in lines using the following conventions:
    - a line ending \r\n or \n is a separator and yields a new list element
    - empty lines or lines with only white spaces are not returned.
    - returned lines are stripped.

    Because of these constraints "".split() cannot be used directly. We first
    replace things where we want to split with line endings, then we
    splitlines.
    """
    # FIXME: leverage new Pythin 3.8 scopeing rules
    return [l.strip() for l in s.splitlines() if l.strip()]


def foldcase(text):
    """
    Fold the case of a text to lower case.
    """
    return text.lower()


def nopunc():
    return re.compile(r'[\W_]', re.MULTILINE | re.UNICODE)


def nopunctuation(text):
    u"""
    Replaces any non alphanum symbol (i.e. punctuation) in text with space.
    Preserve the characters offsets by replacing punctuation with spaces.
    Warning: this also drops line endings.
    """
    if not isinstance(text, str):
        text = as_unicode(text)
    return re.sub(nopunc(), ' ', text)


CR = '\r'
LF = '\n'
CRLF = CR + LF
CRLF_NO_CR = ' ' + LF


def unixlinesep(text, preserve=False):
    """
    Normalize a string to Unix line separators. Preserve character offset by
    replacing with spaces if preserve is True.
    """
    if not isinstance(text, str):
        text = as_unicode(text)
    return text.replace(CRLF, CRLF_NO_CR if preserve else LF).replace(CR, LF)


def nolinesep(text):
    """
    Removes line separators, replacing them with spaces.
    """
    if not isinstance(text, str):
        text = as_unicode(text)
    return text.replace(CR, ' ').replace(LF, ' ')


def toascii(s, translit=False):
    """
    Convert a Unicode or byte string to ASCII characters, including replacing
    accented characters with their non-accented equivalent.

    If `translit` is False use the Unicode NFKD equivalence.
    If `translit` is True, use a transliteration with the unidecode library.

    Non ISO-Latin and non ASCII characters are stripped from the output. When no
    transliteration is possible, the resulting character is replaced by an
    underscore "_".

    For Unicode NFKD equivalence, see http://en.wikipedia.org/wiki/Unicode_equivalence
    The convertion may NOT preserve the original string length and with NFKD some
    characters may be deleted.
    Inspired from: http://code.activestate.com/recipes/251871/#c10 by Aaron Bentley.
    """
    if not isinstance(s, str):
        s = as_unicode(s)
    if translit:
        converted = unidecode(s)
    else:
        converted = unicodedata.normalize('NFKD', s)

    converted = converted.replace('[?]', '_')
    converted = converted.encode('ascii', 'ignore')
    return converted.decode('ascii')


def python_safe_name(s):
    """
    Return a name derived from string `s` safe to use as a Python identifier.
    """
    if not isinstance(s, str):
        s = as_unicode(s)
    s = toascii(s)
    s = foldcase(s)
    s = nopunctuation(s)
    s = s.replace(' ', '_')
    s = '_'.join(s.split())
    s = s.strip('_')
    return s


def as_unicode(s):
    """
    Return a unicode string for a string be it bytes or unicode.
    """
    if isinstance(s, str):
        return s
    if s == b'':
        return u''
    if not s:
        return s
    assert isinstance(s, bytes), 's must be bytes but is: {}'.format(s)
    return UnicodeDammit(s).markup



#
# Copyright (c) nexB Inc. and others. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# See http://www.apache.org/licenses/LICENSE-2.0 for the license text.
# See https://github.com/nexB/commoncode for support or download.
# See https://aboutcode.org for more information about nexB OSS projects.
#

import os

import commoncode.testcase
from commoncode import fileset


class FilesetTest(commoncode.testcase.FileBasedTesting):
    test_data_dir = os.path.join(os.path.dirname(__file__), 'data')

    def test_load(self):
        irf = self.get_test_loc('fileset/scancodeignore.lst')
        result = fileset.load(irf)
        assert result == ['/foo/*', '!/foobar/*', 'bar/*', '#comment']

    def test_is_included_basic(self):
        assert fileset.is_included('/common/src/', {}, {})
        assert fileset.is_included('/common/src/', None, None)
        assert not fileset.is_included(None, None, None)

    def test_is_included_in_fileset(self):
        incs = {'/common/src/*': '.scanignore'}
        excs = {'/common/src/*.so':'.scanignore'}
        assert not fileset.is_included(None, incs, excs)
        assert not fileset.is_included('', incs, excs)
        assert not fileset.is_included('/', incs, excs)
        assert fileset.is_included('/common/src/', incs, excs)
        assert not fileset.is_included('/common/bin/', incs, excs)

    def test_is_included_in_fileset_2(self):
        incs = {'src*': '.scanignore'}
        excs = {'src/ab': '.scanignore'}
        assert not fileset.is_included(None, incs, excs)
        assert not fileset.is_included('', incs, excs)
        assert not fileset.is_included('/', incs, excs)
        assert fileset.is_included('/common/src/', incs, excs)
        assert not fileset.is_included('src/ab', incs, excs)
        assert fileset.is_included('src/abbab', incs, excs)

    def test_is_included_is_included_exclusions(self):
        incs = {'/src/*': '.scanignore'}
        excs = {'/src/*.so':'.scanignore'}
        assert not fileset.is_included('/src/dist/build/mylib.so', incs, excs)

    def test_is_included_is_included_exclusions_2(self):
        incs = {'src': '.scanignore'}
        excs = {'src/*.so':'.scanignore'}
        assert fileset.is_included('/some/src/this/that', incs, excs)
        assert not fileset.is_included('/src/dist/build/mylib.so', incs, excs)

    def test_is_included_empty_exclusions(self):
        incs = {'/src/*': '.scanignore'}
        excs = {'': '.scanignore'}
        assert fileset.is_included('/src/dist/build/mylib.so', incs, excs)

    def test_is_included_sources(self):
        incs = {'/home/elf/elf-0.5/*': '.scanignore'}
        excs = {'/home/elf/elf-0.5/src/elf': '.scanignore',
                '/home/elf/elf-0.5/src/elf.o': '.scanignore'}
        assert not fileset.is_included('/home/elf/elf-0.5/src/elf', incs, excs)

    def test_is_included_dot_svn(self):
        incs = {'*/.svn/*': '.scanignore'}
        excs = {}
        assert fileset.is_included('home/common/tools/elf/.svn/', incs, excs)
        assert fileset.is_included('home/common/tools/.svn/this', incs, excs)
        assert not fileset.is_included('home/common/tools/this', incs, excs)

    def test_is_included_dot_svn_with_excludes(self):
        incs = {'*/.svn/*': '.scanignore'}
        excs = {'*/.git/*': '.scanignore'}
        assert fileset.is_included('home/common/tools/elf/.svn/', incs, excs)
        assert fileset.is_included('home/common/tools/.svn/this', incs, excs)
        assert not fileset.is_included('home/common/.git/this', incs, excs)

    def test_get_matches(self):
        patterns = {'*/.svn/*': '.scanignore'}
        assert fileset.get_matches('home/common/tools/elf/.svn/', patterns)
        assert fileset.get_matches('home/common/tools/.svn/this', patterns)
        assert not fileset.get_matches('home/common/.git/this', patterns)

    def test_get_matches_accepts_a_list_or_tuple(self):
        patterns = ['*/.svn/*']
        assert fileset.get_matches('home/common/tools/elf/.svn/', patterns)

        patterns = '*/.svn/*',
        assert fileset.get_matches('home/common/tools/elf/.svn/', patterns)



#
# Copyright (c) nexB Inc. and others. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# See http://www.apache.org/licenses/LICENSE-2.0 for the license text.
# See https://github.com/nexB/commoncode for support or download.
# See https://aboutcode.org for more information about nexB OSS projects.
#

import functools

from types import GeneratorType
from array import array


def flatten(seq):
    """
    Flatten recursively a sequence and all its sub-sequences that can be tuples,
    lists or generators (generators will be consumed): all are converted to a
    flat list of elements.

    For example::

    >>> flatten([7, (6, [5, [4, ['a'], 3]], 3), 2, 1])
    [7, 6, 5, 4, 'a', 3, 3, 2, 1]
    >>> def gen():
    ...     for i in range(2):
    ...         yield range(5)
    ...
    >>> flatten(gen())
    [0, 1, 2, 3, 4, 0, 1, 2, 3, 4]

    Originally derived from http://www.andreasen.org/misc/util.py
    2002-2005 by Erwin S. Andreasen -- http://www.andreasen.org/misc.shtml
    This file is in the Public Domain
    Version: Id: util.py,v 1.22 2005/12/16 00:08:21 erwin Exp erwin
    """
    r = []
    generators = GeneratorType, range

    for x in seq:
        if isinstance(x, (list, tuple)):
            r.extend(flatten(x))
        elif isinstance(x, generators):
            r.extend(flatten(list(x)))
        else:
            r.append(x)
    return r


def memoize(fun):
    """
    Decorate `fun` function and cache return values. Arguments must be hashable.
    Only args are supported, kwargs are not handled. Used to speed up some often
    executed functions.

    For example::

    >>> @memoize
    ... def expensive(*args, **kwargs):
    ...     print('Calling expensive with', args, kwargs)
    ...     return 'value expensive to compute' + repr(args)
    >>> expensive(1, 2)
    Calling expensive with (1, 2) {}
    'value expensive to compute(1, 2)'
    >>> expensive(1, 2)
    'value expensive to compute(1, 2)'
    >>> expensive(1, 2, a=0)
    Calling expensive with (1, 2) {'a': 0}
    'value expensive to compute(1, 2)'
    >>> expensive(1, 2, a=0)
    Calling expensive with (1, 2) {'a': 0}
    'value expensive to compute(1, 2)'
    >>> expensive(1, 2)
    'value expensive to compute(1, 2)'
    >>> expensive(1, 2, 5)
    Calling expensive with (1, 2, 5) {}
    'value expensive to compute(1, 2, 5)'

    The expensive function returned value will be cached based for each args
    values and computed only once in its life. Call with kwargs are not cached
    """
    memos = {}

    @functools.wraps(fun)
    def memoized(*args, **kwargs):
        # calls with kwargs are not handled and not cached
        if kwargs:
            return fun(*args, **kwargs)
        # convert any list args to a tuple
        args = tuple(tuple(arg) if isinstance(arg, (list, tuple, array)) else arg
                     for arg in args)
        try:
            return memos[args]
        except KeyError:
            memos[args] = fun(*args)
            return memos[args]

    return functools.update_wrapper(memoized, fun)


def partial(func, *args, **kwargs):
    """
    Return a partial function. Same as functools.partial but keeping the __doc__
    and __name__ of the warpper function.
    """
    return functools.update_wrapper(functools.partial(func, *args, **kwargs), func)



#
# Copyright (c) nexB Inc. and others. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# See http://www.apache.org/licenses/LICENSE-2.0 for the license text.
# See https://github.com/nexB/commoncode for support or download.
# See https://aboutcode.org for more information about nexB OSS projects.
#

import os

from commoncode.testcase import FileBasedTesting

from commoncode.hash import b64sha1
from commoncode.hash import checksum
from commoncode.hash import get_hasher
from commoncode.hash import md5
from commoncode.hash import multi_checksums
from commoncode.hash import sha1
from commoncode.hash import sha256
from commoncode.hash import sha512
from commoncode.hash import sha1_git


class TestHash(FileBasedTesting):
    test_data_dir = os.path.join(os.path.dirname(__file__), 'data')

    def test_get_hasher(self):
        h = get_hasher(160)
        assert h(b'a').b64digest() == u'hvfkN_qlp_zhXR3cuerq6jd2Z7g='
        assert h(b'aa').b64digest() == u'4MkDWJjdUvxlxBRUzsnE0mEb-zc='
        assert h(b'aaa').b64digest() == u'fiQN50-x7Qj6CNOAY_amqRRiqBU='

    def test_short_hashes(self):
        h = get_hasher(32)
        assert h(b'a').hexdigest() == u'0cc175b9'
        assert h(b'aa').hexdigest() == u'4124bc0a'
        h = get_hasher(64)
        assert h(b'aa').hexdigest() == u'4124bc0a9335c27f'

    def test_sha1_checksum_on_text(self):
        test_file = self.get_test_loc('hash/dir1/a.txt')
        assert u'3ca69e8d6c234a469d16ac28a4a658c92267c423' == sha1(test_file)

    def test_sha1_checksum_on_text2(self):
        test_file = self.get_test_loc('hash/dir2/a.txt')
        assert u'3ca69e8d6c234a469d16ac28a4a658c92267c423' == sha1(test_file)

    def test_sha1_checksum_on_dos_text(self):
        test_file = self.get_test_loc('hash/dir2/dos.txt')
        assert u'a71718fb198630ae8ba32926015d8555a03cb06c' == sha1(test_file)

    def test_sha1_checksum_base64(self):
        test_file = self.get_test_loc('hash/dir1/a.png')
        assert u'NKxUZdSKmwT8J18JvCIwZg349Pc=' == b64sha1(test_file)

    def test_md5_checksum_on_text(self):
        test_file = self.get_test_loc('hash/dir1/a.txt')
        assert u'40c53c58fdafacc83cfff6ee3d2f6d69' == md5(test_file)

    def test_md5_checksum_on_text2(self):
        test_file = self.get_test_loc('hash/dir2/a.txt')
        assert u'40c53c58fdafacc83cfff6ee3d2f6d69' == md5(test_file)

    def test_md5_checksum_on_dos_text(self):
        test_file = self.get_test_loc('hash/dir2/dos.txt')
        assert u'095f5068940e41df9add5d4cc396c181' == md5(test_file)

    def test_md5_checksum(self):
        test_file = self.get_test_loc('hash/dir1/a.png')
        assert u'4760fb467f1ebf3b0aeace4a3926f1a4' == md5(test_file)

    def test_sha1_checksum(self):
        test_file = self.get_test_loc('hash/dir1/a.png')
        assert u'34ac5465d48a9b04fc275f09bc2230660df8f4f7' == sha1(test_file)

    def test_sha256_checksum(self):
        test_file = self.get_test_loc('hash/dir1/a.png')
        assert u'1b598db6fee8f1ec7bb919c0adf68956f3d20af8c9934a9cf2db52e1347efd35' == sha256(test_file)

    def test_sha512_checksum(self):
        test_file = self.get_test_loc('hash/dir1/a.png')
        assert u'5be9e01cd20ff288fd3c3fc46be5c2747eaa2c526197125330947a95cdb418222176b182a4680f0e435ba8f114363c45a67b30eed9a9222407e63ccbde46d3b4' == sha512(test_file)

    def test_checksum_sha1(self):
        test_file = self.get_test_loc('hash/dir1/a.txt')
        assert '3ca69e8d6c234a469d16ac28a4a658c92267c423' == checksum(test_file, 'sha1')

    def test_checksum_md5(self):
        test_file = self.get_test_loc('hash/dir1/a.txt')
        assert '40c53c58fdafacc83cfff6ee3d2f6d69' == checksum(test_file, 'md5')

    def test_multi_checksums(self):
        test_file = self.get_test_loc('hash/dir1/a.png')
        expected = dict([
            ('md5', u'4760fb467f1ebf3b0aeace4a3926f1a4'),
            ('sha1', u'34ac5465d48a9b04fc275f09bc2230660df8f4f7'),
            ('sha256', u'1b598db6fee8f1ec7bb919c0adf68956f3d20af8c9934a9cf2db52e1347efd35'),
        ])
        result = multi_checksums(test_file, 'md5 sha1 sha256'.split())
        assert result == expected

    def test_multi_checksums_custom(self):
        test_file = self.get_test_loc('hash/dir1/a.png')
        result = multi_checksums(test_file, ('sha512',))
        expected = dict([
            ('sha512', u'5be9e01cd20ff288fd3c3fc46be5c2747eaa2c526197125330947a95cdb418222176b182a4680f0e435ba8f114363c45a67b30eed9a9222407e63ccbde46d3b4'),
        ])
        assert result == expected

    def test_multi_checksums_shattered1(self):
        test_file = self.get_test_loc('hash/sha1-collision/shattered-1.pdf')
        expected = dict([
            ('md5', 'ee4aa52b139d925f8d8884402b0a750c'),
            ('sha1', '38762cf7f55934b34d179ae6a4c80cadccbb7f0a'),
            ('sha256', '2bb787a73e37352f92383abe7e2902936d1059ad9f1ba6daaa9c1e58ee6970d0'),
            ('sha512', '3c19b2cbcf72f7f5b252ea31677b8f2323d6119e49bcc0fb55931d00132385f1e749bb24cbd68c04ac826ae8421802825d3587fe185abf709669bb9693f6b416'),
            ('sha1_git', 'ba9aaa145ccd24ef760cf31c74d8f7ca1a2e47b0'),
        ])
        result = multi_checksums(test_file)
        assert result == expected

    def test_multi_checksums_shattered2(self):
        test_file = self.get_test_loc('hash/sha1-collision/shattered-2.pdf')
        expected = dict([
            ('md5', '5bd9d8cabc46041579a311230539b8d1'),
            ('sha1', '38762cf7f55934b34d179ae6a4c80cadccbb7f0a'),
            ('sha256', 'd4488775d29bdef7993367d541064dbdda50d383f89f0aa13a6ff2e0894ba5ff'),
            ('sha512', 'f39a04842e4b28e04558496beb7cb84654ded9c00b2f873c3ef64f9dfdbc760cd0273b816858ba5b203c0dd71af8b65d6a0c1032e00e48ace0b4705eedcc1bab'),
            # Note: this is not the same as the sha1_git for shattered-1.pdf ;)
            ('sha1_git', 'b621eeccd5c7edac9b7dcba35a8d5afd075e24f2'),
        ])
        result = multi_checksums(test_file)
        assert result == expected

    def test_sha1_git_checksum(self):
        # $ pushd tests/commoncode/data && for f in `find hash/ -type f` ;
        # do echo -n "$f ";git hash-object --literally $f; done && popd
        tests = [t.strip().split() for t in '''
            hash/dir1/a.txt de980441c3ab03a8c07dda1ad27b8a11f39deb1e
            hash/dir1/a.png 5f212358671a3ada8794cb14fb5227f596447a8c
            hash/sha1-collision/shattered-1.pdf ba9aaa145ccd24ef760cf31c74d8f7ca1a2e47b0
            hash/sha1-collision/shattered-2.pdf.ABOUT b332179480cc839f856652be34e1232309d3c077
            hash/sha1-collision/shattered-1.pdf.ABOUT 56a0048f5639b2d14b02f06d26d5c849fba2bc13
            hash/sha1-collision/shattered-2.pdf b621eeccd5c7edac9b7dcba35a8d5afd075e24f2
            hash/dir2/dos.txt 0d2d3a69833f1ebcbf420875cfbc93f132bc8a0b
            hash/dir2/a.txt de980441c3ab03a8c07dda1ad27b8a11f39deb1e
        '''.splitlines() if t.strip()]
        for test_file, expected_sha1_git in tests:
            test_file = self.get_test_loc(test_file)
            # test that we match the git hash-object
            assert sha1_git(test_file) == expected_sha1_git
